@@ -12,7 +12,7 @@ use PPIx::QuoteLike::Constant qw{ @CARP_NOT };
 use PPIx::QuoteLike::Utils qw{ __instance };
 use Scalar::Util ();
 
-our $VERSION = '0.013_01';
+our $VERSION = '0.014_01';
 
 use constant SCALAR_REF	=> ref \0;
 
@@ -100,7 +100,8 @@ sub list {
 	    qw{ type start finish };
 	push @rslt,
 	    join "\t", $self->_class_name( $obj ), $string,
-	    _format_attr( $obj, qw{ encoding failures interpolates } ),
+	    _format_attr( $obj, qw{ encoding failures interpolates
+		indentation } ),
 	    $self->_perl_version( $obj ),
 	    $self->_variables( $obj ),
 	    ;
